/**
 * 
 */
package org.gcube.portlets.user.tdtemplate.client.template.view;

import java.util.ArrayList;
import java.util.List;

import org.gcube.portlets.user.td.gwtservice.shared.tr.ColumnData;
import org.gcube.portlets.user.tdtemplate.client.TdTemplateController;
import org.gcube.portlets.user.tdtemplate.client.event.ExpressionDialogOpenedEvent;
import org.gcube.portlets.user.tdtemplate.client.resources.TdTemplateAbstractResources;
import org.gcube.portlets.user.tdtemplate.client.template.ColumnDefinition;
import org.gcube.portlets.user.tdtemplate.client.template.SetColumnTypeDialogManager;
import org.gcube.portlets.user.tdtemplate.client.template.smart.SmartAddRuleButton;
import org.gcube.portlets.user.tdtemplate.client.template.smart.SmartButtonDescription;
import org.gcube.portlets.user.tdtemplate.client.template.view.external.ExpressionDialogMng;
import org.gcube.portlets.user.tdtemplate.shared.SPECIAL_CATEGORY_TYPE;
import org.gcube.portlets.user.tdtemplate.shared.TdTColumnCategory;
import org.gcube.portlets.user.tdtemplate.shared.TdTDataType;
import org.gcube.portlets.user.tdtemplate.shared.TemplateExpression;

import com.extjs.gxt.ui.client.event.SelectionChangedEvent;
import com.extjs.gxt.ui.client.event.SelectionChangedListener;
import com.extjs.gxt.ui.client.widget.Html;
import com.extjs.gxt.ui.client.widget.Text;
import com.extjs.gxt.ui.client.widget.form.SimpleComboValue;
import com.google.gwt.core.shared.GWT;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.ui.Label;

/**
 * @author Francesco Mangiacrapa francesco.mangiacrapa@isti.cnr.it
 * @Jan 16, 2014
 *
 */
public class ColumnDefinitionView implements ColumnDefinition, UserActionInterface, ColumnElement{
	
	private int columnIndex;
	private Text columnHeaderValue = new Text();
	private boolean isValid = false;
	
	private Label columnDescription = new Label("Not defined");
	private SetColumnTypeDialogManager setColumnTypeDialogManager;
	private TemplatePanel templatePanel;
	
	private ColumnDefinitionView INSTANCE = this;
	
	private SmartAddRuleButton buttonSmartAddRule = new SmartAddRuleButton();
	private ExpressionDialogMng expressionDialogMng;
	
	private List<TemplateExpression> rulesExpression = null;
	
//	private C_ExpressionContainer ruleExpression = null;
	private SPECIAL_CATEGORY_TYPE selectedCategoryType = SPECIAL_CATEGORY_TYPE.NONE;
	
	/**
	 * @param templatePanel 
	 * @param setColumnTypeDialogManager 
	 * @param setColumnTypeDefinition 
	 * 
	 */
	public ColumnDefinitionView(TemplatePanel templatePanel, int index, SetColumnTypeDialogManager setColumnTypeDialogManager) {
		this.templatePanel = templatePanel;
		this.setColumnTypeDialogManager = setColumnTypeDialogManager;
		columnHeaderValue = new Text("Column "+(index+1));
		columnIndex = index;
		initListners();
		setCommandAddRule();
	}

	/**
	 * 
	 */
	private void initListners() {
		
		setColumnTypeDialogManager.getScbCategory().addSelectionChangedListener(new SelectionChangedListener<SimpleComboValue<String>>() {
			
			@Override
			public void selectionChanged(SelectionChangedEvent<SimpleComboValue<String>> se) {
				
				GWT.log("ScbCategory changed");
				
				setIsValid(false);
				resetRule();
				setSpecialCategory();
				validateColumCategoryAndDataType();
				setColumnTypeDialogManager.selectDataTypeIfIsSingle();
				templatePanel.validateTemplate();

			}
			
			private void setSpecialCategory(){
				
				if(setColumnTypeDialogManager.getCategorySelected().getName().compareTo(SPECIAL_CATEGORY_TYPE.DIMENSION.getLabel())==0){
					setSpecialCategoryType(SPECIAL_CATEGORY_TYPE.DIMENSION);
				}else if(setColumnTypeDialogManager.getCategorySelected().getName().compareTo(SPECIAL_CATEGORY_TYPE.TIMEDIMENSION.getLabel())==0){
					setSpecialCategoryType(SPECIAL_CATEGORY_TYPE.TIMEDIMENSION);
				}else if(setColumnTypeDialogManager.getCategorySelected().getName().compareTo(SPECIAL_CATEGORY_TYPE.CODENAME.getLabel())==0){
					setSpecialCategoryType(SPECIAL_CATEGORY_TYPE.CODENAME);
				}else{ 
					setSpecialCategoryType(SPECIAL_CATEGORY_TYPE.NONE);
				}
			}
		});
		
		setColumnTypeDialogManager.getScbDataType().addSelectionChangedListener(new SelectionChangedListener<SimpleComboValue<String>>() {
			

			@Override
			public void selectionChanged(SelectionChangedEvent<SimpleComboValue<String>> se) {
				
				GWT.log("ScbDataType changed: "+selectedCategoryType);
				
				switch (selectedCategoryType) {
					case DIMENSION:
//						templatePanel.setWidgetIntoTable(2, columnIndex, new Html("Choose Reference"));
						templatePanel.setWidgetIntoTable(2, columnIndex, new Html("Rule (unavailable)"));
						break;
						
					case TIMEDIMENSION:
						templatePanel.setWidgetIntoTable(2, columnIndex, new Html("Rule (unavailable)"));
//						buttonSmartAddRule.update("Rule", TdTemplateAbstractResources.dataType());
//						templatePanel.setWidgetIntoTable(2, columnIndex, buttonSmartAddRule);
						break;
						
					default: //ADD EDIT RULE IS DEFAULT BEHAVIOR
						buttonSmartAddRule.update("Rule", TdTemplateAbstractResources.dataType());
						templatePanel.setWidgetIntoTable(2, columnIndex, buttonSmartAddRule);
						break;
				}
				
				updateValidateTemplate();
			}
		});
		
		setColumnTypeDialogManager.getScbReference().addSelectionChangedListener(new SelectionChangedListener<SimpleComboValue<String>>() {
			
			@Override
			public void selectionChanged(SelectionChangedEvent<SimpleComboValue<String>> se) {
				updateValidateTemplate();
			}
		});
		
		setColumnTypeDialogManager.getScbLocales().addSelectionChangedListener(new SelectionChangedListener<SimpleComboValue<String>>() {

			@Override
			public void selectionChanged(SelectionChangedEvent<SimpleComboValue<String>> se) {
				updateValidateTemplate();
			}
		});
		
		setColumnTypeDialogManager.getScbPeriodTypes().addSelectionChangedListener(new SelectionChangedListener<SimpleComboValue<String>>() {

			@Override
			public void selectionChanged(SelectionChangedEvent<SimpleComboValue<String>> se) {
				updateValidateTemplate();
			}
		});
	}
	
	private void updateValidateTemplate(){
		validateColumCategoryAndDataType();
		templatePanel.validateTemplate();
	}
	
	/**
	 * Column Data Type Validating
	 */
	private void validateColumCategoryAndDataType(){
		setIsValid(false);
		
//		boolean validating = setColumnTypeDialogManager.getDataTypeSelected()!=null?true:false;
		boolean validating;
		
		if(setColumnTypeDialogManager.getCategorySelected().getName().compareTo(SPECIAL_CATEGORY_TYPE.DIMENSION.getLabel())==0){
			ColumnData sel = setColumnTypeDialogManager.getReferenceSelected();
//			validating = setColumnTypeDialogManager.getReferenceSelected()!=null?true:false;
//			ColumnData sel = setColumnTypeDialogManager.getReferenceSelected();
			
			validating = sel!=null?true:false;
			if(validating){ //IF COLUMN DATA IS ALREADY SELECTED

				String  trName = setColumnTypeDialogManager.getReferenceTabularResourceName();
				
				String descr = "";
				if(trName!=null){
					descr = trName + " -> ";
				}
				descr+=sel.getLabel();
				
//				templatePanel.clearCell(2, columnIndex); //REMOVE Choose Reference
				updateColumnDescription("Added reference:", descr, false, false);
			}
			
			
		}else if(setColumnTypeDialogManager.getCategorySelected().getName().compareTo(SPECIAL_CATEGORY_TYPE.TIMEDIMENSION.getLabel())==0){
//			validating = setColumnTypeDialogManager.getDataTypeSelected()!=null?true:false;
			validating = setColumnTypeDialogManager.getSelectedPeriodType()!=null?true:false;
		}else if(setColumnTypeDialogManager.getCategorySelected().getName().compareTo(SPECIAL_CATEGORY_TYPE.CODENAME.getLabel())==0){
				validating = setColumnTypeDialogManager.getSelectedLocale()!=null?true:false;
		}else{ //NORMAL BEHAVIOUR
			validating = setColumnTypeDialogManager.getDataTypeSelected()!=null?true:false;
		}
		
		setIsValid(validating);
	}
	
	/**
	 * 
	 */
	private void setCommandAddRule(){
		
		buttonSmartAddRule.setCommand(new Command() {
			@Override
			public void execute() {
				
				try {
				
//					buttonSmartAddRule.panelMask(true);
					if(expressionDialogMng==null){
						String columnId = "Column "+(columnIndex+1);
						expressionDialogMng = new ExpressionDialogMng(columnId, getSelectedColumnCategory().getId(), getSelectedDataType().getName());
					}
				
//					buttonSmartAddRule.panelMask(false);
					
					TdTemplateController.getInternalBus().fireEvent(new ExpressionDialogOpenedEvent(columnIndex));
					
//					updateColumnDescription("test "+columnIndex);
					
					expressionDialogMng.getExpressionDialog().show();

				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		});
	}
	
	public void resetRule(){
		templatePanel.clearCell(2, columnIndex); //IF EXISTS REMOVE EDIT RULE
		templatePanel.clearCell(3, columnIndex); //IF EXISTS REMOVE DESCRIPTION ROW
		expressionDialogMng = null; //RESET RULE DIALOG
	}
	
	public void resetDescriptionCell(){
		templatePanel.clearCell(3, columnIndex); //IF EXISTS REMOVE DESCRIPTION ROW
		expressionDialogMng = null; //RESET RULE DIALOG
		resetRulesExpressions();
	}
	
	public void resetEditRuleCell(){
		templatePanel.clearCell(2, columnIndex); //IF EXISTS REMOVE EDIT RULE
		expressionDialogMng = null; //RESET RULE DIALOG
	}
	
	public void resetRulesExpressions(){
		this.rulesExpression.clear();
	}
	
	public void addRule(TemplateExpression expression, boolean isEditable, boolean isDeletable){

		if(this.rulesExpression==null)
			this.rulesExpression = new ArrayList<TemplateExpression>();
			
		resetRulesExpressions();
		
		this.rulesExpression.add(expression);
			
		updateColumnDescription("Added Rule:", expression.getHumanDescription(), isDeletable, isEditable);
		
	}
	
	public void updateColumnDescription(String title, String descr, boolean isDeletable, boolean isEditable){
		
		SmartButtonDescription button = new SmartButtonDescription(title, descr, this, isDeletable, isEditable);
//		SmartHtmlDiv ruleDescription = new SmartHtmlDiv("Added Rule:", descr, null);
		templatePanel.setWidgetIntoTable(3, columnIndex, button);
	}

	public int getColumnIndex() {
		return columnIndex;
	}

	public void setColumnIndex(int columnIndex) {
		this.columnIndex = columnIndex;
	}

	public Label getColumnDescription() {
		return columnDescription;
	}

	public void setColumnDescription(Label columnDescription) {
		this.columnDescription = columnDescription;
	}

	public Text getColumnHeaderValue() {
		return columnHeaderValue;
	}

	public void setColumnHeaderValue(Text columnHeaderValue) {
		this.columnHeaderValue = columnHeaderValue;
	}

	public SetColumnTypeDialogManager getSetColumnTypeViewManager() {
		return setColumnTypeDialogManager;
	}


	@Override
	public String getColumnName() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public TdTDataType getSelectedDataType() {
		return setColumnTypeDialogManager.getDataTypeSelected();
	}

	@Override
	public boolean isValid() {
		return isValid;
	}
	
	private void setIsValid(boolean bool){
		isValid = bool;
	}

	@Override
	public TdTColumnCategory getSelectedColumnCategory() {
		return setColumnTypeDialogManager.getCategorySelected();
	}

	
	/* (non-Javadoc)
	 * @see org.gcube.portlets.user.tdtemplate.client.template.ColumnDefinition#setCategoryType(org.gcube.portlets.user.tdtemplate.shared.CATEGORY_TYPE)
	 */
	@Override
	public void setSpecialCategoryType(SPECIAL_CATEGORY_TYPE category) {
		GWT.log("SPECIAL_CATEGORY_TYPE setted as: "+category);
		this.selectedCategoryType = category;
		
	}

	/* (non-Javadoc)
	 * @see org.gcube.portlets.user.tdtemplate.client.template.ColumnDefinition#getCategoryType()
	 */
	@Override
	public SPECIAL_CATEGORY_TYPE getSpecialCategoryType() {
		return this.selectedCategoryType;
	}

	/* (non-Javadoc)
	 * @see org.gcube.portlets.user.tdtemplate.client.template.ColumnDefinition#getColumnData()
	 */
	@Override
	public ColumnData getReferenceColumnData() {
		return setColumnTypeDialogManager.getReferenceSelected();
	}

	/**
	 * 
	 */
	@Override
	public String getSelectedLocale() {
		return setColumnTypeDialogManager.getSelectedLocale();
		
	}

	/**
	 * 
	 */
	@Override
	public String getTimePeriod() {
		return setColumnTypeDialogManager.getSelectedPeriodType();
	}

	
	public List<TemplateExpression> getRulesExpression() {
		return rulesExpression;
	}

	public void setRulesExpression(List<TemplateExpression> rulesExpression) {
		this.rulesExpression = rulesExpression;
	}
	
	@Override
	public String getColumnId() {
		return null;
	}
	
	public void setAsReadOnly(boolean enabled){
		
		setColumnTypeDialogManager.getScbCategory().setReadOnly(enabled);
		setColumnTypeDialogManager.getScbDataType().setReadOnly(enabled);
		setColumnTypeDialogManager.getScbLocales().setReadOnly(enabled);
		setColumnTypeDialogManager.getScbPeriodTypes().setReadOnly(enabled);
		setColumnTypeDialogManager.getScbReference().setReadOnly(enabled);
		
//		buttonSmartAddRule.enable(!enabled);
		
	}
	
	
	public void setColumnTypeAsReadOnly(boolean enabled){
		setColumnTypeDialogManager.getScbCategory().setReadOnly(enabled);
		setColumnTypeDialogManager.getScbDataType().setReadOnly(enabled);
	}

	/* (non-Javadoc)
	 * @see org.gcube.portlets.user.tdtemplate.client.template.view.ActionInterface#editClicked()
	 */
	@Override
	public void editClicked() {
	}

	/* (non-Javadoc)
	 * @see org.gcube.portlets.user.tdtemplate.client.template.view.ActionInterface#cancelClicked()
	 */
	@Override
	public void deleteClicked() {
		resetDescriptionCell();
	}

	/* (non-Javadoc)
	 * @see org.gcube.portlets.user.tdtemplate.client.template.view.ColumnElement#getColumnDataType()
	 */
	@Override
	public String getColumnDataType() {
		return this.getSelectedColumnCategory().getName();
	}

	/* (non-Javadoc)
	 * @see org.gcube.portlets.user.tdtemplate.client.template.view.ColumnElement#getColumnType()
	 */
	@Override
	public String getColumnType() {
		return this.getSelectedColumnCategory().getId();
	}

}
